<?php
/**
 * This class manages all functionality with our Capture theme.
 */
class Capture {
	const CAPTURE_VERSION = '1.0.8';

	private static $instance; // Keep track of the instance

	/**
	 * Function used to create instance of class.
	 */
	public static function instance() {
		if ( ! isset( self::$instance ) )
			self::$instance = new Capture;

		return self::$instance;
	}


	/**
	 * This function sets up all of the actions and filters on instance
	 */
	function __construct() {
		add_action( 'after_switch_theme', array( $this, 'after_switch_theme' ) ); // Flush rewrite rules on activation
		add_action( 'after_setup_theme', array( $this, 'after_setup_theme' ), 20 ); // Register image sizes
		add_action( 'widgets_init', array( $this, 'widgets_init' ), 20 ); // Unregister sidebars and alter Primary Sidebar output
		add_action( 'pre_get_posts', array( $this, 'pre_get_posts' ) ); // Used to enqueue editor styles based on post type
		add_action( 'wp_enqueue_scripts', array( $this, 'wp_enqueue_scripts' ) ); // Enqueue all stylesheets (Main Stylesheet, Fonts, etc...)
		add_action( 'the_post', array( $this, 'the_post' ) ); // Remove Default Jetpack Share Buttons
		add_filter( 'the_content', array( $this, 'the_content' ) ); // Prepend Jetpack Share Buttons
		add_action( 'wp_footer', array( $this, 'wp_footer' ) ); // Responsive navigation functionality

		// Theme Customizer
		add_action( 'customize_register', array( $this, 'customize_register' ) ); // Switch background properties to use refresh transport method
		add_filter( 'theme_mod_content_color', array( $this, 'theme_mod_content_color' ) ); // Set the default content color

		// Capture Slideshow
		add_action( 'wp_ajax_capture_slideshow', array( $this, 'wp_ajax_capture_slideshow' ) ); // Ajax/Backbone Requests
		add_action( 'wp_ajax_nopriv_capture_slideshow', array( $this, 'wp_ajax_capture_slideshow' ) ); // Ajax/Backbone Requests
		add_action( 'init', array( $this, 'init' ) ); // URL Endpoint
		add_filter( 'request', array( $this, 'request' ) ); // Modify request for URL Endpoint (Above)

		// Capture Theme Hooks
		add_action( 'capture_post_footer_right', array( $this, 'capture_post_footer_right' ) ); // Output Jetpack Share Buttons

		// WooCommerce
		remove_action( 'woocommerce_before_main_content', 'woocommerce_output_content_wrapper', 10 ); // Remove default WooCommerce content wrapper
		remove_action( 'woocommerce_after_main_content', 'woocommerce_output_content_wrapper_end', 10 ); // Remove default WooCommerce content wrapper
		add_action( 'woocommerce_before_main_content', array( $this, 'woocommerce_before_main_content' ) ); // Add Capture WooCommerce content wrapper
		add_action( 'woocommerce_after_main_content', array( $this, 'woocommerce_after_main_content' ) ); // Add Capture WooCommerce content wrapper
		add_filter( 'woocommerce_catalog_settings', array( $this, 'woocommerce_catalog_settings' ) ); // Adjust default WooCommerce catalog settings
		add_filter( 'loop_shop_per_page', array( $this, 'loop_shop_per_page' ), 20 ); // Adjust number of items displayed on a catalog page
		remove_action( 'woocommerce_after_single_product_summary', 'woocommerce_output_related_products', 20 ); // Remove default WooCommerce related products
		add_action( 'woocommerce_after_single_product_summary', array( $this, 'woocommerce_after_single_product_summary' ), 20 ); // Add WooCommerce related products (3x3)
	}


	/************************************************************************************
	 *    Functions to correspond with actions above (attempting to keep same order)    *
	 ************************************************************************************/

	/**
	 * This function flushes the rewrite rules to ensure the endpoint registered below functions properly.
	 */
	function after_switch_theme() {
		flush_rewrite_rules();
	}

	/**
	 * This function adds images sizes to WordPress.
	 */
	function after_setup_theme() {
		add_image_size( 'capture-1200x500', 1200, 500, true ); // Used for featured images on blog page
		add_image_size( 'capture-1200x9999', 1200, 9999, false ); // Used for featured images on single posts and pages

		// Remove top & footer navigation areas which are registered in SDS Core (Theme Options)
		unregister_nav_menu( 'top_nav' );
		unregister_nav_menu( 'footer_nav' );

		// WooCommerce Support
		add_theme_support( 'woocommerce' );

		// Change default core markup for search form, comment form, and comments, etc... to HTML5
		add_theme_support( 'html5', array(
			'search-form',
			'comment-form',
			'comment-list'
		) );

		// Custom Background (color/image)
		add_theme_support( 'custom-background', array(
			'default-color' => '#ffffff'
		) );
	}

	/**
	 * This function unregisters extra sidebars that are not used in this theme. It also modifies
	 * the global $wp_registered_sidebars to change the output of the Primary Sidebar for the "tuck" effect (CSS).
	 */
	function widgets_init() {
		global $wp_registered_sidebars;

		// Primary Sidebar
		if ( isset( $wp_registered_sidebars['primary-sidebar'] ) ) {
			$wp_registered_sidebars['primary-sidebar']['before_widget'] .= '<section class="widget-container">';
			$wp_registered_sidebars['primary-sidebar']['after_widget'] .= '</section>';
		}

		// Unregister unused sidebars which are registered in SDS Core
		unregister_sidebar( 'front-page-slider-sidebar' );
		unregister_sidebar( 'front-page-sidebar' );
		unregister_sidebar( 'header-call-to-action-sidebar' );
		unregister_sidebar( 'footer-sidebar' );
	}

	/**
	 * This function adds editor styles based on post type, before TinyMCE is initialized.
	 * It will also enqueue the correct color scheme stylesheet to better match front-end display.
	 */
	function pre_get_posts() {
		global $sds_theme_options, $post;

		$protocol = is_ssl() ? 'https' : 'http';

		// Admin only and if we have a post
		if ( is_admin() && ! empty( $post ) ) {
			add_editor_style( 'css/editor-style.css' );

			// Add correct color scheme if selected
			if ( function_exists( 'sds_color_schemes' ) && ! empty( $sds_theme_options['color_scheme'] ) && $sds_theme_options['color_scheme'] !== 'default' ) {
				$color_schemes = sds_color_schemes();
				add_editor_style( 'css/' . $color_schemes[$sds_theme_options['color_scheme']]['stylesheet'] );
			}

			// Open Sans & Oswald Web Fonts (include only if a web font is not selected in Theme Options)
			if ( ! function_exists( 'sds_web_fonts' ) || empty( $sds_theme_options['web_font'] ) )
				add_editor_style( $protocol . '://fonts.googleapis.com/css?family=Open+Sans|Oswald' ); // Google WebFonts (Open Sans & Oswald)
		}
	}


	/**
	 * This function enqueues all styles and scripts (Main Stylesheet, Fonts, etc...).
	 * Stylesheets can be conditionally included if needed
	 */
	function wp_enqueue_scripts() {
		global $sds_theme_options, $post;

		$is_mobile = wp_is_mobile(); // Determine if user is on a mobile device
		$protocol = is_ssl() ? 'https' : 'http'; // Determine current protocol

		// Capture (main stylesheet)
		wp_enqueue_style( 'capture', get_template_directory_uri() . '/style.css', false, self::CAPTURE_VERSION );

		// Enqueue the child theme stylesheet only if a child theme is active
		if ( is_child_theme() )
			wp_enqueue_style( 'capture-child', get_stylesheet_uri(), array( 'capture' ), self::CAPTURE_VERSION );

		// Damion, Open Sans & Oswald Web Fonts (include only if a web font is not selected in Theme Options)
		if ( ! function_exists( 'sds_web_fonts' ) || empty( $sds_theme_options['web_font'] ) )
			wp_enqueue_style( 'damion-open-sans-oswald-web-fonts', $protocol . '://fonts.googleapis.com/css?family=Damion|Open+Sans|Oswald', false, self::CAPTURE_VERSION ); // Google WebFonts (Damion, Open Sans & Oswald)

		// Font Awesome
		wp_enqueue_style( 'font-awesome-css-min', get_template_directory_uri() . '/includes/css/font-awesome.min.css' );

		// Ensure jQuery is loaded on the front end for our footer script (@see wp_footer() below)
		wp_enqueue_script( 'jquery' );

		// If we're not on a single attachment page
		if ( ! is_attachment() ) {
			// ImagesLoaded
			wp_enqueue_script( 'imagesloaded-min', get_template_directory_uri() . '/js/imagesloaded.min.js', array( 'jquery' ), self::CAPTURE_VERSION, true );

			// Hammer
			if ( $is_mobile )
				wp_enqueue_script( 'hammer-min', get_template_directory_uri() . '/js/hammer.min.js', false, self::CAPTURE_VERSION, true );

			// Capture Slideshow
			wp_enqueue_script( 'capture-slideshow-min', get_template_directory_uri() . '/js/capture-slideshow.min.js', ( $is_mobile ) ? array( 'jquery', 'backbone', 'hammer-min' ) : array( 'jquery', 'backbone' ), self::CAPTURE_VERSION, true );
			wp_localize_script( 'capture-slideshow-min', 'capture', array(
				'ajaxurl' => admin_url( 'admin-ajax.php' ),
				'nonce' => wp_create_nonce( 'capture_slideshow' ),
				'permalinks' => ( get_option( 'permalink_structure' ) ) ? true : false,
				'is_archive' => ( is_archive() || is_home() || is_search() ) ? true : false,
				'is_mobile' => ( $is_mobile ) ? true : false
			) );
		}
	}

	/**
	 * This function removes the default Jetpack Share Buttons output on single posts and
	 * posts shown on the home, archive, and search page templates.
	 */
	function the_post( $post ) {
		if ( class_exists( 'Jetpack' ) && Jetpack::is_module_active( 'sharedaddy' ) && $post->post_type === 'post' && ( is_single() || is_home() || is_archive() || is_search() ) ) {
			remove_filter( 'the_content', 'sharing_display', 19 );
			remove_filter( 'the_excerpt', 'sharing_display', 19 );
		}
	}

	/**
	 * This function prepends Jetpack Share Buttons to single post content.
	 */
	function the_content( $content ) {
		if ( class_exists( 'Jetpack' ) && Jetpack::is_module_active( 'sharedaddy' ) && is_single() )
			$content = sharing_display() . $content;

		return $content;
	}

	/**
	 * This function outputs the necessary Javascript for the responsive menus and Capture Slideshow.
	 */
	function wp_footer() {
		global $post;
	?>
		<script type="text/javascript">
			// <![CDATA[
				jQuery( function( $ ) {
					// Mobile Nav
					$( '.mobile-nav-button' ).on( 'touch click', function ( e ) {
						e.stopPropagation();
						$( '.mobile-nav-button, .mobile-nav, body' ).toggleClass( 'open' );
					} );

					$( '.mobile-nav-close-button' ).on( 'touch click', function ( e ) {
						e.stopPropagation();
						$( '.mobile-nav-button, .mobile-nav, body' ).removeClass( 'open' );
					} );

					$( document ).on( 'touch click', function() {
						$( '.mobile-nav-button, .mobile-nav, body' ).removeClass( 'open' );
					} );

					<?php if ( is_single() && has_post_thumbnail() ) : // Single Posts with Featured Images ?>
						$( '.post-image-full-link', '.post' ).click( function( e ) {
							e.preventDefault();

							$( '#capture-slideshow' ).captureSlideshow(); // Capture Slideshow
						} );
					<?php endif; ?>
				} );

				<?php if ( is_single() && has_post_thumbnail() && get_query_var( 'lightbox' ) ) : // Determine if the request contains lightbox query and enable slideshow if so ?>
					jQuery( window ).load( function() {
						jQuery( '#capture-slideshow' ).captureSlideshow( {
							lightbox: true
						} );
					} );
				<?php endif; ?>
			// ]]>
		</script>
	<?php
	}


	/********************
	 * Theme Customizer *
	 ********************/

	/**
	 * This function sets background-color and background-image settings in Theme Customizer to use
	 * the refresh method instead of postMessage.
	 */
	function customize_register( $wp_customize ) {
		if ( get_theme_support( 'custom-background', 'wp-head-callback' ) === '_custom_background_cb' ) {
			foreach ( array( 'color', 'image' ) as $prop ) {
				$wp_customize->get_setting( 'background_' . $prop )->transport = 'refresh';
			}
		}
	}

	/**
	 * This function sets the default color for the content area in the Theme Customizer.
	 */
	function theme_mod_content_color( $color ) {
		return $color ? $color : '#333333';
	}


	/**
	 * This function handles Ajax/Backbone requests for the Capture Slideshow
	 */
	function wp_ajax_capture_slideshow() {
		global $wpdb;

		// Make sure we have a valid request
		check_ajax_referer( 'capture_slideshow', 'nonce' );

		// Determine protocol
		$protocol = is_ssl() ? 'https' : 'http';

		// Select two previous, the current, and two next posts with featured images
		$current_post_id = ( isset( $_REQUEST['current_post_id'] ) ) ? abs( ( int ) $_REQUEST['current_post_id'] ) : false;
		$initial_post_id = ( isset( $_REQUEST['initial_post_id'] ) ) ? abs( ( int ) $_REQUEST['initial_post_id'] ) : false;
		$num_next_prev_posts = ( isset( $_REQUEST['num_next_prev_posts'] ) && ( int ) $_REQUEST['num_next_prev_posts'] <= 5 ) ? abs( ( int ) $_REQUEST['num_next_prev_posts'] ) : 2;

		if ( $current_post_id && $initial_post_id && $num_next_prev_posts ) {
			$capture_slideshow_posts = $wpdb->get_results( "( SELECT ID, post_title, post_date FROM $wpdb->posts INNER JOIN $wpdb->postmeta ON ( $wpdb->posts.ID = $wpdb->postmeta.post_id ) WHERE post_date < ( SELECT post_date FROM $wpdb->posts WHERE ID = $current_post_id ) AND ( $wpdb->posts.post_type = 'post' ) AND ( $wpdb->posts.post_status = 'publish' ) AND ( $wpdb->postmeta.meta_key = '_thumbnail_id' ) GROUP BY $wpdb->posts.ID ORDER BY post_date DESC LIMIT $num_next_prev_posts ) UNION ( SELECT ID, post_title, post_date FROM $wpdb->posts INNER JOIN $wpdb->postmeta ON ( $wpdb->posts.ID = $wpdb->postmeta.post_id ) WHERE ID = $current_post_id OR post_date > ( SELECT post_date AS current_post_date FROM $wpdb->posts WHERE ID = $current_post_id ) AND ( $wpdb->posts.post_type = 'post' ) AND ( $wpdb->posts.post_status = 'publish' ) AND ( $wpdb->postmeta.meta_key = '_thumbnail_id' ) GROUP BY $wpdb->posts.ID ORDER BY post_date LIMIT " . ( $num_next_prev_posts + 1 ) . " )" );

			// Build Backbone Models (if we have posts)
			if ( ! empty( $capture_slideshow_posts ) && is_array( $capture_slideshow_posts ) ) {
				$models = array();
				$current_image_index = 0;

				// Ensure we have an array ordered by post date DESC
				usort( $capture_slideshow_posts, function( $a, $b ) {
					return strnatcmp( strtotime( $b->post_date ), strtotime( $a->post_date ) );
				} );

				foreach( $capture_slideshow_posts as $post ) {
					$featured_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'full' );

					// Capture Slideshow Model
					$model = array(
						'img' => array(
							'src' => $featured_image[0],
							'width' => $featured_image[1],
							'height' => $featured_image[2]
						),
						'post' => array(
							'id' => $post->ID,
							'title' => $post->post_title,
							'date' => sprintf( __( 'Posted On %1$s', 'capture' ) , get_the_time( 'F jS, Y', $post->ID ) ),
							'timestamp' => strtotime( $post->post_date ),
							'permalink' => str_replace( $protocol . '://' . $_SERVER['SERVER_NAME'], '', get_permalink( $post->ID ) ) // Just return the URI
						),
					);

					// Append current post details to Capture Slideshow Model
					if ( ( int ) $post->ID === $current_post_id ) {
						$model['current_image'] = true;
						$current_image_index = count( $models );
					}

					// Append initial post details to Capture Slideshow Model
					if ( ( int ) $post->ID === $initial_post_id )
						$model['initial_image'] = true;

					$models[] = $model; // Append current model to model group
				}

				// Check to see if there are no more previous/next images
				if ( count( $models ) < ( $num_next_prev_posts * 2 + 1 ) ) {
					// No more previous images (current image is not in the middle)
					if ( $current_image_index < $num_next_prev_posts )
						$models[0]['no_more_images'] = true;

					// No more next images
					if( ( count( $models ) - $current_image_index - 1 ) < $num_next_prev_posts )
						$models[( count( $models ) - 1 )]['no_more_images'] = true;
				}

				// Check the database if necessary just to make sure
				if ( count( $models ) <= ( $num_next_prev_posts * 2 + 1 ) ) {
					if ( ! isset( $models[0]['no_more_images'] ) && ! ( $no_more_images = $wpdb->get_var( "SELECT COUNT(*) FROM $wpdb->posts INNER JOIN $wpdb->postmeta ON ( $wpdb->posts.ID = $wpdb->postmeta.post_id ) WHERE $wpdb->posts.post_date > ( SELECT post_date FROM $wpdb->posts WHERE ID = " . $models[0]['post']['id'] . " ) AND ( $wpdb->posts.post_type = 'post' ) AND ( $wpdb->posts.post_status = 'publish' ) AND ( $wpdb->postmeta.meta_key = '_thumbnail_id' )" ) ) )
						$models[0]['no_more_images'] = true;
					if ( ! isset( $models[( count( $models ) - 1 )]['no_more_images'] ) && ! ( $no_more_images = $wpdb->get_var( "SELECT COUNT(*) FROM $wpdb->posts INNER JOIN $wpdb->postmeta ON ( $wpdb->posts.ID = $wpdb->postmeta.post_id ) WHERE post_date < ( SELECT post_date FROM $wpdb->posts WHERE ID = " . $models[( count( $models ) - 1 )]['post']['id'] . " ) AND ( $wpdb->posts.post_type = 'post' ) AND ( $wpdb->posts.post_status = 'publish' ) AND ( $wpdb->postmeta.meta_key = '_thumbnail_id' )" ) ) )
						$models[( count( $models ) - 1 )]['no_more_images'] = true;
				}

				echo json_encode( $models ); // Return models to collection
			}
		}

		exit;
	}

	/**
	 * This function adds a URL Endpoint for the Capture Slideshow lightbox.
	 */
	function init() {
		add_rewrite_endpoint( 'lightbox', EP_PERMALINK ); // http://example.com/permalink/lightbox/
	}

	/**
	 * This function modifies the request to ensure the above URL Endpoint functions as we desire.
	 */
	 function request( $request ) {
		// Check to make sure our endpoint is set
		if( isset( $request['lightbox'] ) )
			$request['lightbox'] = true;

		return $request;
	}


	/***********************
	 * Capture Theme Hooks *
	 ***********************/

	/**
	 * This function outputs Jetpack Share Buttons in post footers.
	 */
	function capture_post_footer_right( $post ){
		if( class_exists( 'Jetpack' ) && Jetpack::is_module_active( 'sharedaddy' ) && $post->post_type === 'post' )
			echo sharing_display();
	}


	/***************
	 * WooCommerce *
	 ***************/

	/**
	 * This function alters the default WooCommerce content wrapper starting element.
	 */
	function woocommerce_before_main_content(){
	?>
		<section class="woocommerce woo-commerce post">
			<section class="post-container">
				<article class="post-content cf">
	<?php
	}

	/**
	 * This function alters the default WooCommerce content wrapper ending element.
	 */
	function woocommerce_after_main_content(){
	?>
				</article>
			</section>
		</section>
	<?php
	}


	/**
	 * This function adjusts the default WooCommerce Catalog settings.
	 */
	function woocommerce_catalog_settings( $settings ) {
		if ( is_array( $settings ) ) {
			foreach( $settings as &$setting ) {
				// Adjust the default value of the Catalog image size
				if( $setting['id'] === 'shop_catalog_image_size' ) {
					$setting['default']['width'] = $setting['default']['height'] = 300;
				}
			}
		}

		return $settings;
	}

	/**
	 * This function changes the number of products output on the Catalog page.
	 */
	function loop_shop_per_page( $num_items ) {
		return 12;
	}

	/**
	 * This function changes the number of related products displayed on a single product page.
	 */
	function woocommerce_after_single_product_summary() {
		woocommerce_related_products( 3, 3 );
	}
}


function CaptureInstance() {
	return Capture::instance();
}

// Starts Capture
CaptureInstance();